<?
class KBLib {
	var $db; // The PEAR db object used to access the database

	function KBLib($db,$tikilib)
	{
		if(!$db) {
			die("Invalid db object passed to UsersLib (kblib) constructor");
		}
		$this->db = $db;
		$this->tikilib = $tikilib;
	}
	function sql_error($query, $result)
	{
		trigger_error("MYSQL error: ".$result->getMessage()." in query:<br/>".$query."<br/>",E_USER_WARNING);
		die;
	}

	
	//
	// *** Library support ***
	//
	
	
	/**
	 * Get the areas of documents that are searchable
	 *
	 * This function returns a hardcoded list of fields that are searchable
	 * in any supported document types. As new capabilities are added to other
	 * library routines, this function's list should be updated.
	 *
	 * This function is useful when creating future-proof HTML via smarty -
	 * just get Smarty to iterate through this list and generate your HTML
	 */	
	function get_supported_areas()
	{
		// internally, these get truncated to two characters
		// so use names that are unique, even when they're only
		// two characters long ;-)
		$supported=array("Title","Description");
		
		return $supported;
	}
	
	// get a list of supported types, or test something is supported
	// If changing this, be aware the database has a table which is
	// written in the same order - ie. item 1 in this list has ID 1 in
	// the database. So, if changing this, append only to avoid DB
	// problems for people "in the field".
	/**
	 * Get the document types that are supported by this library
	 *
	 * This function returns a hard coded list of document types that
	 * are supported by this library. Also, this function can be used
	 * to check if a particular document type is supported.
	 *
	 * This function aids writing future-proof code elsewhere, and
	 * also helps sanitise inputs as support for an input type can
	 * be checked easily.
	 */
	function get_supported_doctypes($doctype)
	{
		$supported=array("article","faq", "wiki", "kbdoc", "kbfaq");
		
		if(isset($doctype))
		{
			// check if the given doctype is supported
			foreach ($supported as $item)
			{
				if($doctype == $item)
				{
					return true;
				}
			}
			return false;
		}
		return $supported;
	}
	

	// 
	// *** Misc stuff ***
	//

	// sanitise_input just strips any non-numerics from input and returns the result
	// useful for making usre numbers are actually numbers, which is handy when
	// trying to avoid SQL injection attacks ;-)
	/**
	 * Clean up numeric input data to avoid SQL injection problems
	 *
	 * This function takes a numeric input in either a string or a numeric
	 * and "cleans" it, removing any non-numeric characters and returning
	 * a numeric value. It only works with integers, but ensures that
	 * user input that is supposed to be numeric really is numeric. This
	 * can be useful to stop SQL injection attacks in otherwise sleepy code.
	 */
	function sanitise_input($input)
	{
		if(!isset($input))
			return;
		$input=preg_replace('/\D/','',$input);
		$input = $input + 0;
		return $input;
	}

	// this one's deprecated ;-)
	function get_doc_attribs($assignmentid)
	{
		$assignmentid=$this->sanitise_input($assignmentid);
		$query = "select * from tiki_kb_assignments where assignmentID=$assignmentid";
		$result = $this->db->query($query);
		
		if(DB::isError($result)) $this->sql_error($query, $result);
		$ret = Array();
		$res = $result->fetchRow(DB_FETCHMODE_ASSOC);
		
		$query2 = "select * from tiki_kb_categories where categoryid=$res[category1]";
		$result2 = $this->db->query($query2);
		$res2 = $result2->fetchRow(DB_FETCHMODE_ASSOC);
		$cat1=$res2['categoryname'];;
			
		$query2 = "select * from tiki_kb_categories where categoryid=$res[category2]";
		$result2 = $this->db->query($query2);
		$res2 = $result2->fetchRow(DB_FETCHMODE_ASSOC);
		$cat2=$res2['categoryname'];
		
		$keywords=$res['keywords'];
								
		return array($cat1,$cat2,$keywords);
	}
	
	/**
	 * Get a document assignment row from the database
	 *
	 * Fetches a row from the database with the supplied assignment ID. Also,
	 * populates extra fields that do not exist in the DB with the relational
	 * data that the row contains. For example, the row contains a document type
	 * ID. This number is meaningless to most things, so this function fetches
	 * the document type and also the display name for that doctype. Naturally,
	 * these have more uses in the world.
	 *
	 * Fields returned:
	 * assignmentID, doctype, category1, category2, keywords, docID, wikiname
	 * doctype_typeid, displayname
	 *
	 * Note, category1, category2 and doctype are returned as the english names,
	 * not the IDs that are actually stored in the database tables.
	 */
	function get_assignment_row($assignmentid)
	{
		$assignmentid=$this->sanitise_input($assignmentid);
		$query = "select * from tiki_kb_assignments where assignmentID=$assignmentid";
		$result = $this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		
		$ret = Array();
		$res = $result->fetchRow(DB_FETCHMODE_ASSOC);
		
		if(!isset($res['assignmentID']))
			return false;
		
		$query2 = "select * from tiki_kb_categories where categoryid=$res[category1]";
		$result2 = $this->db->query($query2);
		if(DB::isError($result)) $this->sql_error($query, $result2);
		$res2 = $result2->fetchRow(DB_FETCHMODE_ASSOC);
		$res['category1']=$res2['categoryname'];;
			
		$query2 = "select * from tiki_kb_categories where categoryid=$res[category2]";
		$result2 = $this->db->query($query2);
		if(DB::isError($result)) $this->sql_error($query, $result2);
		$res2 = $result2->fetchRow(DB_FETCHMODE_ASSOC);
		$res['category2']=$res2['categoryname'];
		
		$query2 = "select * from tiki_kb_doctypes where typeid=$res[doctype]";
		$result2 = $this->db->query($query2);
		if(DB::isError($result)) $this->sql_error($query, $result2);
		$res2 = $result2->fetchRow(DB_FETCHMODE_ASSOC);
		$res['doctype_typeid']=$res['doctype'];
		$res['doctype']= $res2['doctype'];
		$res['displayname']= $res2['displayname'];
								
		return $res;
	}	
	
	// deprecated - use get_assignment_row instead
	function get_doc_type($assignmentid)
	{
		$assignmentid=$this->sanitise_input($assignmentid);
		
		$query = "select * from tiki_kb_assignments where assignmentID=$assignmentid";
		$result = $this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		$res = $result->fetchRow(DB_FETCHMODE_ASSOC);
		
		$query2 = "select * from tiki_kb_doctypes where typeid=$res[doctype]";
		$result2 = $this->db->query($query2);
		$res2 = $result2->fetchRow(DB_FETCHMODE_ASSOC);
		$plain_doctype= $res2['doctype'];
		$display_doctype= $res2['displayname'];
		
		return array($plain_doctype,$display_doctype);
	}
		
	// deprecated - use get_assignment_row instead
	function get_doc_number($assignmentid)
	{
		$assignmentid=$this->sanitise_input($assignmentid);
		$query = "select * from tiki_kb_assignments where assignmentID=$assignmentid";
		$result = $this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		$res = $result->fetchRow(DB_FETCHMODE_ASSOC);
		
		return array($res['docID'],$res['wikiname']);
	}
		
	// used by browse function
	/**
	 * get a list of KB documents given a doctype id
	 *
	 * Given a doctype id, this function returns a list of assignment rows
	 * of documents with that doctype. Primarily for use in browsing.
	 */
	function get_kb_assignments_by_doctype($doctype)
	{
		$docid=$this->sanitise_input($doctype);
		
		$query="select * from tiki_kb_assignments where doctype=$doctype";
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
	
		$results=array();
	
		while($res = $result->fetchRow(DB_FETCHMODE_ASSOC))
		{
			$results[]=$res;
		}
		
		return $results;
	}
	
	// used by browse to get a title of a doc, no matter what type it is
	/**
	 * get a document title, given it's assignment row
	 *
	 * This function looks at the assignment row it is given and fetches
	 * the title of the document to which that relates. In most cases, this
	 * requires an extra DB lookup. In the case of Wikis, it actually just
	 * returns one of the fields in the row it's been given(!).
	 * Either way, the caller gets the document title back, no matter what
	 * kind of document it was.
	 */
	function get_doc_title_by_assignment($assignment_row)
	{
		// First, figure out what kind of document it is...
		$doctype_row=$this->get_doctype_by_id($assignment_row['doctype']);
		
		// Now, switch on the document type, and fetch what ever we need...
		switch ($doctype_row['doctype'])
		{
			case "article":
				$query="select title from tiki_articles where articleId=".$assignment_row['docID'];
				$title=$this->db->getOne($query);
				return $title;
				break;
			case "faq":
				$query="select title from tiki_faqs where faqID=".$assignment_row['docID'];
				$title=$this->db->getOne($query);
				return $title;
				break;
			case "wiki":
				return $assignment_row['wikiname'];
				break;
			case "kbdoc":
				$query="select title from tiki_kb_documents where docID=".$assignment_row['docID'];
				$title=$this->db->getOne($query);
				return $title;
				break;
			case "kbfaq":
				$query="select title from tiki_kbfaqs where faqID=".$assignment_row['docID'];
				$title=$this->db->getOne($query);
				return $title;
				break;
		}
	
		// Shouldn't ever get here!
		return "";
	}
	
	
	// used by assign existing documents
	/**
	 * get a list of KB assignments given a document ID (not an assignment ID!)
	 *
	 * Given a document ID (which is far from unique), get a list of assignments
	 * that have that document ID. This list is then used to whittle down the
	 * list of documents available to categorise. This is done by checking
	 * doctypes and then seeing if it's the sort of doctype required (thus
	 * unique-ifying the docid).
	 *
	 * In the case of Wiki pages, the wikiname is actually unique, so no further
	 * work is required.
	 *
	 * In either case, this routine just blindy returns stuff to the caller.
	 * It's really pretty simple ;-)
	 */
	function get_kb_assignments_by_docid($docid,$wikiname)
	{
		if(isset($wikiname))
		{
			$query="select * from tiki_kb_assignments where wikiname=$wikiname";
		}
		else
		{
			$docid=$this->sanitise_input($docid);
			$query="select * from tiki_kb_assignments where docID=$docid";
		}
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
	
		$results=array();
	
		while($res = $result->fetchRow(DB_FETCHMODE_ASSOC))
		{
			$results[]=$res;
		}
		
		return $results;
	}
	
	
	//
	// *** DocType Handling
	//
	
	/**
	 * Get the doctype displaynames available on the system
	 *
	 * Fetch a list of displaynames of the doctypes that are configured
	 * in the system.
	 */
	function get_available_doctypes()
	{
		$doctypes=array();
		
		$query = "select * from tiki_kb_doctypes";
		$result = $this->db->query($query);
		
		if(DB::isError($result)) $this->sql_error($query, $result);
		
		while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
			$doctypes[]=$res['displayname'];
		}
		
		return $doctypes;
	}

	
	/**
	 * Get Doctypes Table from the database
	 *
	 * This returns an array of associative arrays containing
	 * "typeid", "doctype" and "displayname". The whole
	 * table is returned.
	 */
	function get_doctypes_table()
	{
		$doctypes=array();
		
		$query = "select * from tiki_kb_doctypes";
		$result = $this->db->query($query);
		
		if(DB::isError($result)) $this->sql_error($query, $result);
		
		while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
			$doctypes[]=$res;
		}
		
		return $doctypes;
	}
	
	function get_doctype_by_id($typeid)
	{
		$typeid=$this->sanitise_input($typeid);
		$row=array();
		
		$query="select * from tiki_kb_doctypes where typeid=$typeid";
		$result=$this->db->query($query);
		
		if(DB::isError($result)) $this->sql_error($query, $result);
		
		$row=$result->fetchRow(DB_FETCHMODE_ASSOC);
						
		return $row;
	}
	
	function get_doctype_by_name($displayname)
	{
		$row=array();
		
		$query="select * from tiki_kb_doctypes where displayname='".$displayname."'";
		$result=$this->db->query($query);
		
		if(DB::isError($result)) $this->sql_error($query, $result);
		
		$row=$result->fetchRow(DB_FETCHMODE_ASSOC);
						
		return $row;
	}
	
	function change_doctype_name($typeid,$newname)
	{
		$typeid=$this->sanitise_input($typeid);
		$newname=addslashes(strip_tags($newname,''));
		
		// First, see if it's already in the database
		$query="select * from tiki_kb_doctypes where typeid=$typeid";
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		
		$row=$result->fetchRow(DB_FETCHMODE_ASSOC);
	
		$query="update tiki_kb_doctypes set displayname='".$newname."' where typeid=$typeid";
		$result=$this->db->query($query);
			
		if(DB::isError($result)) $this->sql_error($query, $result);
		//if(DB::isError($result)) return false;
	
		return true;
	}

	function change_doctype($doctype,$typeid)
	{
		$typeid=$this->sanitise_input($typeid);
		if(!$this->get_supported_doctypes($doctype))
			return false;
		$query="update tiki_kb_doctypes set doctype='".$doctype."' where typeid=$typeid";
		$result=$this->db->query($query);
		//if(DB::isError($result)) $this->sql_error($query, $result);
		if(DB::isError($result)) return false;
	
		return true;
	}

	function add_new_doctype($displayname,$doctype)
	{
		if(!$this->get_supported_doctypes($doctype))
			return false;
		$displayname=addslashes(strip_tags($displayname,''));
		$query="insert into tiki_kb_doctypes (doctype, displayname) values ('".$doctype."', '".$displayname."')";
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		if(DB::isError($result)) return false;
	
		return true;
	}
	
	function delete_doctype($typeid)
	{
		$typeid=$this->sanitise_input($typeid);
		// First, have a look to see if there are documents of this type...
		$query="select * from tiki_kb_assignments where doctype=$typeid";
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		
		$row=$result->fetchRow(DB_FETCHMODE_ASSOC);
		if(isset($row['doctype']))
		{
			// we got at least one row with stuff in it - we can't delete
			return false;
		}
		
		// First, get the displayname of the thing we're about to delete...
		$details=$this->get_doctype_by_id($typeid);
		
		// Delete the row...
		$query="delete from tiki_kb_doctypes where typeid=$typeid";
		$result = $this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
			
		return $details['displayname'];
	}
	
	

	//
	// *** Category Handling
	//
	
	
	function get_available_categories($level)
	{
		$level=$this->sanitise_input($level);
		$out=array();
		
		$query = "select * from tiki_kb_categories where level=$level";
		$result = $this->db->query($query);
		
		if(DB::isError($result)) $this->sql_error($query, $result);
		while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
			$out[]=$res['categoryname'];
		}
		
		return $out;
	}

	function get_categories($level)
	{
		$level=$this->sanitise_input($level);
	
		$cats=array();
		
		$query = "select * from tiki_kb_categories where level=$level";
		$result = $this->db->query($query);
		
		if(DB::isError($result)) $this->sql_error($query, $result);
		while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
			$cats[]=$res;
		}
		
		return $cats;
	}
	
	function get_category_by_id($catid)
	{
		$catid=$this->sanitise_input($catid);
		$row=array();
		
		$query="select * from tiki_kb_categories where categoryid=$catid";
		$result=$this->db->query($query);
		
		if(DB::isError($result)) $this->sql_error($query, $result);
		
		$row=$result->fetchRow(DB_FETCHMODE_ASSOC);
				
		return $row;
	}
	
	function get_category_by_name($catname,$level)
	{
		$row=array();
		
		if(isset($level))
			$mid="level=$level and";
		else
			$mid="";
			
		$catname=addslashes(strip_tags($catname,''));
		
		$query="select * from tiki_kb_categories where $mid categoryname='$catname'";
		$result=$this->db->query($query);
		
		if(DB::isError($result)) $this->sql_error($query, $result);
		
		$row=$result->fetchRow(DB_FETCHMODE_ASSOC);
				
		return $row;
	}
	
	function change_category_name($catid,$newname)
	{
		$catid=$this->sanitise_input($catid);
		$newname=addslashes(strip_tags($newname,''));
		$query="update tiki_kb_categories set categoryname='".$newname."' where categoryid=$catid";
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//if(DB::isError($result)) return false;
	
		return true;
	}

	function add_new_category($catname,$level)
	{
		$level=$this->sanitise_input($level);
		
		$catname=addslashes(strip_tags($catname,''));
		
		// First, see if it's already in there...
		$query="select * from tiki_kb_categories where level=$level and categoryname='$catname'";
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		$row=$result->fetchRow(DB_FETCHMODE_ASSOC);
		if((isset($row['categoryname'])) && ($row['categoryname'] != ""))
			// category already exists - skip it silently
			return true;

		
		$query="insert into tiki_kb_categories (level, categoryname) values ($level, '".$catname."')";
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//if(DB::isError($result)) return false;
	
		return true;
	}
	
	function delete_category($catid)
	{
		$catid=$this->sanitise_input($catid);
		
		// First, have a look to see if there are documents of this category...
		$result=$this->db->query("select * from tiki_kb_assignments where category1=$catid");
		$row=$result->fetchRow(DB_FETCHMODE_ASSOC);
		if(isset($row['doctype']))
		{
			// we got at least one row with stuff in it - we can't delete
			return false;
		}
		$result=$this->db->query("select * from tiki_kb_assignments where category2=$catid");
		$row=$result->fetchRow(DB_FETCHMODE_ASSOC);
		if(isset($row['doctype']))
		{
			// we got at least one row with stuff in it - we can't delete
			return false;
		}
		
		// First, get the displayname of the thing we're about to delete...
		$details=$this->get_category_by_id($catid);
		
		// Delete the row...
		$query="delete from tiki_kb_categories where categoryid=$catid";
		$result = $this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
			
		return $details['categoryname'];
	}

	//
	// *** KB Doc Handling ***
	//
	
	function get_kb_document($docid)
	{
		$docid=$this->sanitise_input($docid);
		
		$query="select * from tiki_kb_documents where docID=$docid";
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
	
		$row=$result->fetchRow(DB_FETCHMODE_ASSOC);
		
		
		$row['reads']=$row['reads'] + 1;
		
		$query="update tiki_kb_documents set reads=".$row['reads']." where docID=$docid";
		$result=$this->db->query($query);
		// don't care if that doesn't work though ;-)
		
		return $row;
	}

	function add_new_kb_document($title,$author,$desc,$body,$authorisation)
	{
		if($authorisation == "on")
			$authorisation='y';
		elseif($authorisation == "")
			$authorisation='n';
			
		if(($authorisation != 'y') && ($authorisation != 'n'))
			$authorisation='n';
			
		// This should be set to "now"
		$date=date("U");

		$title=addslashes(strip_tags($title,''));
		$desc=addslashes(strip_tags($desc,''));
		$body=addslashes(strip_tags($body,'<a>'));

		$query="insert into tiki_kb_documents (title,author,date,description,body,reads,authflag) values ";
		$query="$query ('$title', '$author',$date,'$desc','$body',0,'$authorisation')";
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//if(DB::isError($result)) return false;
	
		// now find the document we just added...
		$query="select docID from tiki_kb_documents where ";
		$query="$query (title='$title' and author='$author' and date=$date and description='$desc' and body='$body' and reads=0 and authflag='$authorisation')";
		$result=$this->db->getOne($query);
	
		if(!isset($result))
			return false;
	
		return $result;
	}

	function update_kb_document($docid,$title,$author,$desc,$body,$authorisation)
	{
		if($authorisation == "on")
			$authorisation='y';
		elseif($authorisation == "")
			$authorisation='n';
			
		if(($authorisation != 'y') && ($authorisation != 'n'))
			$authorisation='n';
			
		// This should be set to "now"
		$date=date("U");
		
		$title=addslashes(strip_tags($title,''));
		$desc=addslashes(strip_tags($desc,''));
		$body=addslashes(strip_tags($body,'<a>'));
		
		$query="update tiki_kb_documents set title='$title',author='$author',date=$date,description='$desc',body='$body',authflag='$authorisation' where docID=$docid";
			
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//if(DB::isError($result)) return false;
	
		return true;
	}
	
	
	function delete_kb_document($docid)
	{
		$query="delete from tiki_kb_documents where docid=$docid";
			
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//if(DB::isError($result)) return false;
	
		return true;
	}
	
	//
	// *** Native FAQ handling ***
	//
	// Pretty much just lifted from faqlib.php and tikilib.php
	// the database table names have been changed to protect the innocent ;-)
	//

	function add_suggested_faq_question($docId,$question, $answer, $user)
	{
		// Additional bit - get the faqId from the docId...
		
		$row=$this->get_assignment_row($docId);
		$faqId=$row['docID'];
		
		$question = addslashes(strip_tags($question,'<a>'));
		$answer = addslashes(strip_tags($answer,'<a>'));
		$now= date("U");
		$query = "insert into tiki_kb_suggested_faq_questions(faqId,question,answer,user,created)
		values($faqId,'$question','$answer','$user',$now)";
		$result = $this->db->query($query);
	}
	
  function list_suggested_questions($offset,$maxRecords,$sort_mode,$find)
  {
    $sort_mode = str_replace("_"," ",$sort_mode);
    if($find) {
      $mid=" where (question like '%".$find."%' or answer like '%".$find."%')";
    } else {
      $mid="";
    }
    $query = "select * from tiki_kb_suggested_faq_questions $mid order by $sort_mode limit $offset,$maxRecords";
    $query_cant = "select count(*) from tiki_kb_suggested_faq_questions $mid";
    $result = $this->db->query($query);
    $cant = $this->db->getOne($query_cant);
    $ret = Array();
    while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
      $ret[] = $res;
    }
    $retval = Array();
    $retval["data"] = $ret;
    $retval["cant"] = $cant;
    return $retval;
  }  

  function list_all_faq_questions($offset,$maxRecords,$sort_mode,$find)
  {
    $sort_mode = str_replace("_"," ",$sort_mode);
    if($find) {
      $mid=" where (question like '%".$find."%' or answer like '%".$find."%')";
    } else {
      $mid="";
    }
    $query = "select * from tiki_kbfaq_questions $mid order by $sort_mode limit $offset,$maxRecords";
    $query_cant = "select count(*) from tiki_kbfaq_questions $mid";
    $result = $this->db->query($query);
    $cant = $this->db->getOne($query_cant);
    $ret = Array();
    while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
      $ret[] = $res;
    }
    $retval = Array();
    $retval["data"] = $ret;
    $retval["cant"] = $cant;
    return $retval;
  }
  
  function remove_faq($faqId)
  {
    $query = "delete from tiki_kbfaqs where faqId=$faqId";
    $result = $this->db->query($query);
    $query = "delete from tiki_kbfaq_questions where faqId=$faqId";
    $result = $this->db->query($query);
    // Remove comments and/or individual permissions for faqs
    $this->tikilib->remove_object('faq',$faqId);
    return true;
  }
  
  function remove_faq_question($questionId)
  {
    $faqId=$this->db->getOne("select faqId from tiki_kbfaq_questions where questionId=$questionId");
    $query = "delete from tiki_kbfaq_questions where questionId=$questionId";
    $result = $this->db->query($query);
    $query = "update tiki_kbfaqs set questions=questions-1 where faqId=$faqId";
    $result = $this->db->query($query);
    return true;
  }

  function get_faq_question($questionId)
  {
    $query = "select * from tiki_kbfaq_questions where questionId=$questionId";
    $result = $this->db->query($query);
    if(!$result->numRows()) return false;
    $res = $result->fetchRow(DB_FETCHMODE_ASSOC);
    return $res;
  }
  
  function add_faq_hit($faqId)
  {
    $query = "update tiki_kbfaqs set hits=hits+1 where faqId=$faqId";
    $result = $this->db->query($query);
  }

  function replace_faq_question($faqId,$questionId, $question, $answer)
  {
    $question = addslashes($question);
    $answer = addslashes($answer);
    // Check the name

    if((isset($questionId)) && ($questionId != "")) {
      $query = "update tiki_kbfaq_questions set question='$question',answer='$answer' where questionId=$questionId";
    } else {
      $query = "update tiki_kbfaqs set questions=questions+1 where faqId=$faqId";
      $result = $this->db->query($query);
      if(DB::isError($result)) $this->sql_error($query, $result);
      $query = "replace into tiki_kbfaq_questions(faqId,question,answer)
                values($faqId,'$question','$answer')";
    }

    $result = $this->db->query($query);
    
    if(DB::isError($result)) $this->sql_error($query, $result);
	//if(DB::isError($result)) return false;
	
    return true;
  }

  function replace_faq($faqId, $title, $description,$canSuggest)
  {
  	$faqId=$this->sanitise_input($faqId);
    $description = addslashes($description);
    $title = addslashes($title);
    // Check the name

    if((isset($faqId)) && ($faqId != "")) {
      $query = "update tiki_kbfaqs set title='$title',description='$description',canSuggest='$canSuggest' where faqId=$faqId";
      $result = $this->db->query($query);
      if(DB::isError($result)) $this->sql_error($query, $result);
	//if(DB::isError($result)) return false;
    } else {
      $now = date("U");
      $query = "replace into tiki_kbfaqs(title,description,created,hits,questions,canSuggest)
                values('$title','$description',$now,0,0,'$canSuggest')";
      $result = $this->db->query($query);
      if(DB::isError($result)) $this->sql_error($query, $result);
	//if(DB::isError($result)) return false;
      $faqId = $this->db->getOne("select max(faqId) from tiki_kbfaqs where title='$title' and created=$now");
    }
    return $faqId;
  }
  
  function list_faq_questions($faqId,$offset,$maxRecords,$sort_mode,$find)
  {
    $sort_mode = str_replace("_"," ",$sort_mode);
    if($find) {
      $mid=" where faqId=$faqId and (question like '%".$find."%' or answer like '%".$find."%')";
    } else {
      $mid=" where faqId=$faqId ";
    }
    $query = "select * from tiki_kbfaq_questions $mid order by $sort_mode limit $offset,$maxRecords";
    $query_cant = "select count(*) from tiki_kbfaq_questions $mid";
    $result = $this->db->query($query);
    $cant = $this->db->getOne($query_cant);
    $ret = Array();
    while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
      $res['parsed'] = $this->tikilib->parse_data($res['answer']);
      $ret[] = $res;
    }
    $retval = Array();
    $retval["data"] = $ret;
    $retval["cant"] = $cant;
    return $retval;
  }


  function remove_suggested_question($sfqId)
  {
    $query = "delete from tiki_kb_suggested_faq_questions where sfqId=$sfqId";
    $result = $this->db->query($query);
  }

  function approve_suggested_question($sfqId)
  {
    $info = $this->get_suggested_question($sfqId);
    $this->replace_faq_question($info["faqId"],0, $info["question"], $info["answer"]);
    $this->remove_suggested_question($sfqId);
  }
  
	function get_suggested_question($sfqId)
	{
	  $query = "select * from tiki_kb_suggested_faq_questions where sfqId=$sfqId";
	  $result = $this->db->query($query);
	  if(!$result->numRows()) return false;
	  $res = $result->fetchRow(DB_FETCHMODE_ASSOC);
	  return $res;
	}


	function list_faqs($offset,$maxRecords,$sort_mode,$find)
	{
	    $sort_mode = str_replace("_"," ",$sort_mode);
	    if($find) {
	      $mid=" where (title like '%".$find."%' or description like '%".$find."%')";
	    } else {
	      $mid="";
	    }
	    $query = "select * from tiki_kbfaqs $mid order by $sort_mode limit $offset,$maxRecords";
	    $query_cant = "select count(*) from tiki_kbfaqs $mid";
	    $result = $this->db->query($query);
	    $cant = $this->db->getOne($query_cant);
	    $ret = Array();
	    while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
	      $res["suggested"]=$this->db->getOne("select count(*) from tiki_kb_suggested_faq_questions where faqId=".$res["faqId"]);
	      $ret[] = $res;
	    }
	    $retval = Array();
	    $retval["data"] = $ret;
	    $retval["cant"] = $cant;
	    return $retval;
	}
  
	function get_kb_faq($faqId)
	{
	    $query = "select * from tiki_kbfaqs where faqId=$faqId";
	    $result = $this->db->query($query);
	    if(!$result->numRows()) return false;
	    $res = $result->fetchRow(DB_FETCHMODE_ASSOC);
	    return $res;
	}
	
	//
	// *** Assignment handlers ***
	//
	
	function add_new_assignment($doctype,$category1,$category2,$keywords,$docid,$wikiname)
	{
		$doctype=$this->sanitise_input($doctype);
		$category1=$this->sanitise_input($category1);
		$category2=$this->sanitise_input($category2);
		$docid=$this->sanitise_input($docid);
		
		$keywords=addslashes(strip_tags($keywords,''));
		$wikiname=addslashes(strip_tags($wikiname,''));
		
		if(!isset($wikiname))
			$wikiname="";
			
		$keywords = addslashes(strip_tags($keywords,''));
		$wikiname = addslashes(strip_tags($wikiname,''));
			
		$query="insert into tiki_kb_assignments (doctype,category1,category2,keywords,docID,wikiname) values ";
		$query="$query ($doctype,$category1,$category2,'$keywords',$docid,'$wikiname')";
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//if(DB::isError($result)) return false;
	
		// now find the document we just added...
		$query="select assignmentID from tiki_kb_assignments where";
		$query="$query (doctype=$doctype and category1=$category1 and category2=$category2 and keywords='$keywords' and docID=$docid and wikiname='$wikiname')";
		$result=$this->db->getOne($query);
	
		if(!isset($result))
			return false;
	
		return $result;
	
	}
	
	function update_assignment($assignmentid,$doctype,$category1,$category2,$keywords,$docid,$wikiname)
	{
		$doctype=$this->sanitise_input($doctype);
		$category1=$this->sanitise_input($category1);
		$category2=$this->sanitise_input($category2);
		$docid=$this->sanitise_input($docid);
		$assignmentid=$this->sanitise_input($assignmentid);
		if(!isset($wikiname))
			$wikiname="";
			
		$keywords=addslashes(strip_tags($keywords,''));
		$wikiname=addslashes(strip_tags($wikiname,''));
			
		$query="update tiki_kb_assignments set doctype=$doctype,category1=$category1,category2=$category2,keywords='$keywords',docID=$docid,wikiname='$wikiname' where assignmentID=$assignmentid";
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//if(DB::isError($result)) return false;
		
		return true;
	
	}

	function delete_assignment($assignmentid)
	{
		$assignmentid=$this->sanitise_input($assignmentid);

		$query="delete from tiki_kb_assignments where assignmentID=$assignmentid";
		$result=$this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//if(DB::isError($result)) return false;
		
		return true;	
	}
	
	
	//
	// *** Document Finders ***
	//
	
	
	function _find_faqs($typeid,$terms,$areas,$offset,$maxRecords)
	{
		$faq_results=array();
		
		// These aren't yet implemented
		$maxRecords=10;
		$offset=0;
		
		$sort_mode="title";
		
		if($terms) {
			$mid="";
			if((isset($areas["Title"])) && ($areas["Title"] == 1))
			{
				$mid="title like '%".$terms."%'";
			}
			if((isset($areas["Description"])) && ($areas["Description"] == 1))
			{
				if($mid == '')
					$mid="description like '%".$terms."%'";
				else
					$mid="$mid or description like '%".$terms."%'";
			}
			//$mid=" where (title like '%".$terms."%' or description like '%".$terms."%')";
			if($mid != "")
				$mid="where ($mid)";
		} else {
			$mid="";
		}
		
		$query = "select * from tiki_faqs $mid order by $sort_mode limit $offset,$maxRecords";
		//$query_cant = "select count(*) from tiki_kbfaqs $mid";
		$result = $this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//$cant = $this->db->getOne($query_cant);
		
		$display_doctype=$this->db->getOne("select displayname from tiki_kb_doctypes where typeid=$typeid");
		
		while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
			$res["suggested"]=$this->db->getOne("select count(*) from tiki_suggested_faq_questions where faqId=".$res["faqId"]);
			$result2 = $this->db->query("select * from tiki_kb_assignments where docid=".$res["faqId"]." and docType=$typeid");
			if(DB::isError($result)) $this->sql_error($query, $result);
			$res2 = $result2->fetchRow(DB_FETCHMODE_ASSOC);
			
			if(isset($res2["assignmentID"]))
			{
				$res["docid"]=$res2["assignmentID"];
				$res["category1"]=$this->db->getOne("select categoryname from tiki_kb_categories where categoryID=".$res2["category1"]);
				$res["category2"]=$this->db->getOne("select categoryname from tiki_kb_categories where categoryID=".$res2["category2"]);
				$res["keywords"]=$res2["keywords"];
				$res["reads"]=$res["hits"];
				$res["doctype"]=$display_doctype;
				
				// Now, work on body and keyword results. We have to look into each document to do this
				// For FAQs we have to look at the questions and answers for this FAQ document
			
				// Keyword checking
				// NOT IMPLEMENTED
				
			
				// Now do body (meaning questions and answers)
				// NOT IMPLEMENTED		
			
			}
			
			// If it's got a DocID, then we add it to the list the user
			// will get to see. If the docID has been nuked by the stuff
			// above us, then we don't show the user this document.
			// It'll show up in the admin screens as "orphaned" or some such.
			if(isset($res['docid']))
				$faq_results[] = $res;
		}
		return $faq_results;
	}
	
	function _find_articles($typeid,$terms,$areas,$offset,$maxRecords)
	{
		$article_results=array();
		
		// These aren't yet implemented
		$maxRecords=10;
		$offset=0;
		
		$sort_mode="title";

		if($terms) {
			$mid="";
			if((isset($areas["Title"])) && ($areas["Title"] == 1))
			{
				$mid="title like '%".$terms."%'";
			}
			if((isset($areas["Description"])) && ($areas["Description"] == 1))
			{
				if($mid == '')
					$mid="heading like '%".$terms."%'";
				else
					$mid="$mid or heading like '%".$terms."%'";
			}
			if((isset($areas["body"])) && ($areas["body"] == 1))
			{
				if($mid == '')
					$mid="body like '%".$terms."%'";
				else
					$mid="$mid or body like '%".$terms."%'";
			}
			//$mid=" where (title like '%".$terms."%' or description like '%".$terms."%')";
			if($mid != "")
				$mid="where ($mid)";
		} else {
			$mid="";
		}
		
		$query = "select * from tiki_articles $mid order by $sort_mode limit $offset,$maxRecords";
		//$query_cant = "select count(*) from tiki_articles $mid";
		$result = $this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//$cant = $this->db->getOne($query_cant);
		
		$display_doctype=$this->db->getOne("select displayname from tiki_kb_doctypes where typeid=$typeid");
		
		while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
			$result2 = $this->db->query("select * from tiki_kb_assignments where docid=".$res["articleId"]." and docType=$typeid");
			if(DB::isError($result)) $this->sql_error($query, $result);
			$res2 = $result2->fetchRow(DB_FETCHMODE_ASSOC);
			if(DB::isError($result)) $this->sql_error($query, $result);
			
			if(isset($res2["assignmentID"]))
			{
			
				$res["docid"]=$res2["assignmentID"];
				$res["category1"]=$this->db->getOne("select categoryname from tiki_kb_categories where categoryID=".$res2["category1"]);
				$res["category2"]=$this->db->getOne("select categoryname from tiki_kb_categories where categoryID=".$res2["category2"]);
				$res["keywords"]=$res2["keywords"];
				$res["doctype"]=$display_doctype;
				$res["description"]=$res["heading"];

				// Now, work on keyword results.
				// Keyword checking
				// NOT IMPLEMENTED				
			}
			
			// If it's got a DocID, then we add it to the list the user
			// will get to see. If the docID has been nuked by the stuff
			// above us, then we don't show the user this document.
			// It'll show up in the admin screens as "orphaned" or some such.
			if(isset($res["docid"]))
				$article_results[] = $res;
		}
	
		return $article_results;
	}
	
	function _find_wikis($typeid,$terms,$areas,$offset,$maxRecords)
	{
		$wiki_results=array();
		
		// These aren't yet implemented
		$maxRecords=10;
		$offset=0;
		
		$sort_mode="pageName";

		if($terms) {
			$mid="";
			if((isset($areas["Title"])) && ($areas["Title"] == 1))
			{
				$mid="pageName like '%".$terms."%'";
			}
			if((isset($areas["Description"])) && ($areas["Description"] == 1))
			{
				if($mid == '')
					$mid="description like '%".$terms."%'";
				else
					$mid="$mid or description like '%".$terms."%'";
			}
			if((isset($areas["body"])) && ($areas["body"] == 1))
			{
				if($mid == '')
					$mid="data like '%".$terms."%'";
				else
					$mid="$mid or data like '%".$terms."%'";
			}
			//$mid=" where (title like '%".$terms."%' or description like '%".$terms."%')";
			if($mid != "")
				$mid="where ($mid)";
		} else {
			$mid="";
		}
		
		$query = "select * from tiki_pages $mid order by $sort_mode limit $offset,$maxRecords";
		//$query_cant = "select count(*) from tiki_pages $mid";
		$result = $this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//$cant = $this->db->getOne($query_cant);
		
		$display_doctype=$this->db->getOne("select displayname from tiki_kb_doctypes where typeid=$typeid");
		
		while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
			$result2 = $this->db->query("select * from tiki_kb_assignments where wikiname='".$res["pageName"]."' and docType=$typeid");
			if(DB::isError($result)) $this->sql_error("[query]", $result);
			
			$res2 = $result2->fetchRow(DB_FETCHMODE_ASSOC);
			if(isset($res2["assignmentID"]))
			{
			
				$res["docid"]=$res2["assignmentID"];
				$res["category1"]=$this->db->getOne("select categoryname from tiki_kb_categories where categoryID=".$res2["category1"]);
				$res["category2"]=$this->db->getOne("select categoryname from tiki_kb_categories where categoryID=".$res2["category2"]);
				$res["keywords"]=$res2["keywords"];
				$res["doctype"]=$display_doctype;
				$res["description"]=$res["description"];
				$res["reads"]=$res["hits"];
				$res["created"]=$res["lastModif"];

				// Now, work on keyword results.
				// Keyword checking
				// NOT IMPLEMENTED				
			}
			
			// If it's got a DocID, then we add it to the list the user
			// will get to see. If the docID has been nuked by the stuff
			// above us, then we don't show the user this document.
			// It'll show up in the admin screens as "orphaned" or some such.
			if(isset($res["docid"]))
				$wiki_results[] = $res;
		}
	
		return $wiki_results;
	}
	
	
	function _find_kbdocs($typeid,$terms,$areas,$offset,$maxRecords)
	{
		$kbdoc_results=array();
		
		// These aren't yet implemented
		$maxRecords=10;
		$offset=0;
		
		$sort_mode="title";

		if($terms) {
			$mid="";
			if((isset($areas["Title"])) && ($areas["Title"] == 1))
			{
				$mid="title like '%".$terms."%'";
			}
			if((isset($areas["Description"])) && ($areas["Description"] == 1))
			{
				if($mid == '')
					$mid="description like '%".$terms."%'";
				else
					$mid="$mid or description like '%".$terms."%'";
			}
			if((isset($areas["body"])) && ($areas["body"] == 1))
			{
				if($mid == '')
					$mid="body like '%".$terms."%'";
				else
					$mid="$mid or body like '%".$terms."%'";
			}
			//$mid=" where (title like '%".$terms."%' or description like '%".$terms."%')";
			if($mid != "")
				$mid="where ($mid)";
		} else {
			$mid="";
		}
		
		$query = "select * from tiki_kb_documents $mid order by $sort_mode limit $offset,$maxRecords";
		//$query_cant = "select count(*) from tiki_kb_documents $mid";
		$result = $this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//$cant = $this->db->getOne($query_cant);
		
		$display_doctype=$this->db->getOne("select displayname from tiki_kb_doctypes where typeid=$typeid");
		
		while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
			$result2 = $this->db->query("select * from tiki_kb_assignments where docID=".$res['docID']." and docType=$typeid");
			if(DB::isError($result2)) $this->sql_error("[query]", $result2);
			$res2 = $result2->fetchRow(DB_FETCHMODE_ASSOC);
			
			if(isset($res2["assignmentID"]))
			{
			
				$res["docid"]=$res2["assignmentID"];
				$res["category1"]=$this->db->getOne("select categoryname from tiki_kb_categories where categoryID=".$res2["category1"]);
				$res["category2"]=$this->db->getOne("select categoryname from tiki_kb_categories where categoryID=".$res2["category2"]);
				$res["keywords"]=$res2['keywords'];
				$res["doctype"]=$display_doctype;
				$res["description"]=$res['description'];
				$res["created"]=$res['date'];

				// Now, work on keyword results.
				// Keyword checking
				// NOT IMPLEMENTED				
			}
			
			// If it's got a DocID, then we add it to the list the user
			// will get to see. If the docID has been nuked by the stuff
			// above us, then we don't show the user this document.
			// It'll show up in the admin screens as "orphaned" or some such.
			if(isset($res["docid"]))
				$kbdoc_results[] = $res;
		}
	
		return $kbdoc_results;
	}
	
	
	function _find_kbfaqs($typeid,$terms,$areas,$offset,$maxRecords)
	{
		$faq_results=array();
		
		// These aren't yet implemented
		$maxRecords=10;
		$offset=0;
		
		$sort_mode="title";
		
		if($terms) {
			$mid="";
			if((isset($areas["Title"])) && ($areas["Title"] == 1))
			{
				$mid="title like '%".$terms."%'";
			}
			if((isset($areas["Description"])) && ($areas["Description"] == 1))
			{
				if($mid == '')
					$mid="description like '%".$terms."%'";
				else
					$mid="$mid or description like '%".$terms."%'";
			}
			//$mid=" where (title like '%".$terms."%' or description like '%".$terms."%')";
			if($mid != "")
				$mid="where ($mid)";
		} else {
			$mid="";
		}
		
		$query = "select * from tiki_kbfaqs $mid order by $sort_mode limit $offset,$maxRecords";
		//$query_cant = "select count(*) from tiki_kbfaqs $mid";
		$result = $this->db->query($query);
		if(DB::isError($result)) $this->sql_error($query, $result);
		//$cant = $this->db->getOne($query_cant);
		
		$display_doctype=$this->db->getOne("select displayname from tiki_kb_doctypes where typeid=$typeid");
		
		while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
			$res["suggested"]=$this->db->getOne("select count(*) from tiki_kb_suggested_faq_questions where faqId=".$res["faqID"]);
			$result2 = $this->db->query("select * from tiki_kb_assignments where docid=".$res["faqID"]." and docType=$typeid");
			if(DB::isError($result)) $this->sql_error($query, $result);
			$res2 = $result2->fetchRow(DB_FETCHMODE_ASSOC);
			
			if(isset($res2["assignmentID"]))
			{
				$res["docid"]=$res2["assignmentID"];
				$res["category1"]=$this->db->getOne("select categoryname from tiki_kb_categories where categoryID=".$res2["category1"]);
				$res["category2"]=$this->db->getOne("select categoryname from tiki_kb_categories where categoryID=".$res2["category2"]);
				$res["keywords"]=$res2["keywords"];
				$res["reads"]=$res["hits"];
				$res["doctype"]=$display_doctype;
				//$res["doctype"]=$this->db->getOne("select displayname from tiki_kb_doctypes where typeid=5");
			
				// Now, work on body and keyword results. We have to look into each document to do this
				// For FAQs we have to look at the questions and answers for this FAQ document
			
				// Keyword checking
				// NOT IMPLEMENTED
				
			
				// Now do body (meaning questions and answers)
				// NOT IMPLEMENTED		
			
			}
			
			// If it's got a DocID, then we add it to the list the user
			// will get to see. If the docID has been nuked by the stuff
			// above us, then we don't show the user this document.
			// It'll show up in the admin screens as "orphaned" or some such.
			if(isset($res['faqID']))
				$faq_results[] = $res;
		}
		return $faq_results;
	}	
	
	
	

	
	// Searches FAQs and Articles for something and returns a complex list.
	// The list returned is a associative array, the members of which are
	// defined by the tiki_kb_doctypes table in the database. Each entry
	// is another array of the actual results.
	function find_docs($terms,$areas,$doctypes,$offset,$maxRecords)
	{
		$results=array();
		$faq_results=array();
		$article_results=array();
		$sort_mode="title";
		
		// These aren't yet implemented
		$maxRecords=10;
		$offset=0;
	
		// santise input to avoid SQL injections problems
		$offset=$this->sanitise_input($offset);
		$maxRecords=$this->sanitise_input($maxRecords);
		
		// we search the database for each type of document.
		// The system only has FAQs and articles, but we can
		// call them anything we like, so we may do multiple
		// searches over the articles (for example) to get
		// "tech docs", "resolutions" and "white papers".
		// This could be extended to include bug reports and the like
		// too.
		// This list should be configurable, so the user can say
		// "search 'tech docs' but not 'bugs'.
		
		$query = "select * from tiki_kb_doctypes";
		$result = $this->db->query($query);
		
		if(DB::isError($result)) $this->sql_error($query, $result);

		while($res = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
			$name=$res['displayname'];
			
			if(isset($doctypes[$name]))
			{
			
				switch ($res['doctype'])
				{
					case "article":
						// Now do article searching...
						$loop_results=$this->_find_articles($res['typeid'],$terms,$areas,$offset,$maxRecords);
						break;
					case "faq":
						// Get FAQ results...
						$loop_results=$this->_find_faqs($res['typeid'],$terms,$areas,$offset,$maxRecords);
						break;
					case "wiki":
						// Get Wiki results
						$loop_results=$this->_find_wikis($res['typeid'],$terms,$areas,$offset,$maxRecords);
						break;
					case "kbdoc":
						// Get KB document results...
						$loop_results=$this->_find_kbdocs($res['typeid'],$terms,$areas,$offset,$maxRecords);
						break;
					case "kbfaq":
						// Get KB document results...
						$loop_results=$this->_find_kbfaqs($res['typeid'],$terms,$areas,$offset,$maxRecords);
						break;
				}
				// melde the results together
				$results[]=$loop_results;
			}
		}
		
		return $results;
	}
}
?>